-- DATABASE STRUCTURE CONVERSION FROM 5.5 (Revision 1) TO 5.7 (Revision 2) --

-- Make changes from pre-released revision (5.5.9-5.5.11) to Revision 1:
ALTER TABLE ic_systemsettings ALTER COLUMN paramval TYPE TEXT;
ALTER TABLE ic_groups ALTER COLUMN groupname TYPE VARCHAR(255);
ALTER TABLE ic_messagetypes ALTER COLUMN typename TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN userlogin TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN userpassword TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN username TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN timezone TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN locale TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN email TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN phonelocal TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN phoneext TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN phonemob TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN fax TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN userposition TYPE VARCHAR(255);

-- truncate big user pictures
UPDATE ic_users SET picture=NULL WHERE picture IS NOT NULL AND char_length(picture)>65535;
ALTER TABLE ic_users ALTER COLUMN picture TYPE VARCHAR(65535);

ALTER TABLE ic_users ALTER COLUMN ipint TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN ipext TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN hostname TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN dnsname TYPE VARCHAR(255);
ALTER TABLE ic_users ALTER COLUMN clientversion TYPE VARCHAR(255);
ALTER TABLE ic_files ALTER COLUMN savedto TYPE VARCHAR(65535);
ALTER TABLE ic_recipientlists ALTER COLUMN listname TYPE VARCHAR(255);
DROP SEQUENCE IF EXISTS ic_seq_groups;
DROP SEQUENCE IF EXISTS ic_seq_messagetypes;
DROP SEQUENCE IF EXISTS ic_seq_users;
DROP SEQUENCE IF EXISTS ic_seq_messages;
DROP SEQUENCE IF EXISTS ic_seq_recipientlists;
DROP TRIGGER IF EXISTS ic_filetype_check_trigger ON ic_messagetypes;
DROP FUNCTION IF EXISTS ic_filetype_check();

-- Extend database structure:
ALTER TABLE ic_recipients ADD COLUMN processed BOOLEAN NOT NULL DEFAULT FALSE;
CREATE INDEX ic_recipients_processed_index ON ic_recipients (processed);
ALTER TABLE ic_recipients ADD COLUMN pcomment VARCHAR(65535);
ALTER TABLE ic_recipients ADD COLUMN pdate BIGINT;

-- For every non-file row of ic_messages set processed=TRUE and pdate=mdate for 
-- all the messages with id<lastreadmsgid. This query could be VERY long in time!
UPDATE 
ic_recipients 
SET 
processed=TRUE, 
pdate=ic_messages.mdate 
FROM 
ic_messages, ic_users 
WHERE 
ic_recipients.rid=ic_users.id AND 
ic_recipients.mid<=ic_users.lastreadmsgid AND 
ic_recipients.mid=ic_messages.id AND 
ic_messages.tid<>(SELECT id FROM ic_messagetypes WHERE templatename='IC_FILES' LIMIT 1);

-- Copy data about processed files from ic_files to ic_recipients:
UPDATE 
ic_recipients 
SET 
processed=TRUE, 
pdate=ic_files.fdate, 
pcomment=ic_files.savedto 
FROM 
ic_files 
WHERE 
ic_recipients.mid=ic_files.mid AND 
ic_recipients.rid=ic_files.rid;

-- Remove unneeded data:
DROP TABLE ic_files;
ALTER TABLE ic_users DROP COLUMN lastreadmsgid;

-- Add new necessary tables:
CREATE TABLE ic_ldap (
    id INTEGER UNIQUE NOT NULL,
    ldapdomain VARCHAR(255) UNIQUE NOT NULL,
    ldapurl VARCHAR(255) NOT NULL,
    ldapprincipalformat VARCHAR(255) NOT NULL,
    ldapcnattribute VARCHAR(255) NOT NULL,
    ldaploginattribute VARCHAR(255) NOT NULL,
    ldapmappingattribute VARCHAR(255) NOT NULL,
    ldapisbinaryattribute BOOLEAN NOT NULL DEFAULT FALSE,
    ldapsearchbase VARCHAR(255) NOT NULL,
    ldapfilter VARCHAR(255) NOT NULL,
    PRIMARY KEY (id)
);

CREATE TABLE ic_auth_internal (
    uid INTEGER UNIQUE NOT NULL REFERENCES ic_users (id) ON DELETE CASCADE ON UPDATE CASCADE,
    userlogin VARCHAR(255) UNIQUE NOT NULL,
    userpassword VARCHAR(255) NOT NULL,
    PRIMARY KEY (userlogin)
);

CREATE TABLE ic_auth_external (
    uid INTEGER UNIQUE NOT NULL REFERENCES ic_users (id) ON DELETE CASCADE ON UPDATE CASCADE,
    ldapmapping VARCHAR(255) NOT NULL,
    lid INTEGER NOT NULL REFERENCES ic_ldap (id),
    PRIMARY KEY (ldapmapping, lid)
);

CREATE TABLE ic_messages_binarydata (
    mid BIGINT NOT NULL REFERENCES ic_messages (id),
    chunk_id BIGINT NOT NULL,
    chunk_data BYTEA NOT NULL,
    PRIMARY KEY (mid, chunk_id)
);

CREATE INDEX ic_messages_binarydata_mid_index ON ic_messages_binarydata (mid);

CREATE TABLE ic_parameters (
    uid INTEGER NOT NULL REFERENCES ic_users (id) ON UPDATE CASCADE ON DELETE CASCADE,
    paramname VARCHAR(255) NOT NULL,
    paramval TEXT NOT NULL,
    PRIMARY KEY (uid, paramname)
);

-- Copy existing user data from ic_users to ic_auth_internal:
INSERT INTO ic_auth_internal (uid, userlogin, userpassword) SELECT id, userlogin, userpassword FROM ic_users;

-- Remove unneeded data:
ALTER TABLE ic_users DROP COLUMN userlogin;
ALTER TABLE ic_users DROP COLUMN userpassword;

-- DATABASE STRUCTURE CONVERTION IS FINISHED!!! --
